function Event()
{
}

/**
 * This method provides a browser neutral way of adding event listeners
 * to an element.  If the element supports the "addEventListener" method
 * then this method is used.  Otherwise, if the element supports the
 * "attachEvent" method then this method is used.  There are no other fallback
 * methods so if neither of these methods exist then the event listener
 * is not attached.
 * 
 * @param {HTMLElement} elem the element to which an event listener
 *          is being attached (e.g. window, document, document.body)
 * @param {String} event the name of the event WITHOUT the "on"
 *          (e.g. "click", "resize", "mouseup")
 * @param {function} listenerFunc the event listener function
 * @param {Boolean} capture should event capturing be used?
 *          (optional, default = false)
 */
Event.observe = function(elem, event, listenerFunc, capture)
{
    // "ready" is a special value that we map to dojo/ready
    if (event === 'ready') {
        require(['dojo/ready'], function(ready) {
            ready(function() {
                listenerFunc.call(window);
            });
        });
        return;
    }
    
    if (elem.addEventListener)
    {
        if (!capture)
        {
            capture = false;
        }
        
        elem.addEventListener(event, listenerFunc, capture);
        return;
    }
    
    if (elem.attachEvent)
    {
        elem.attachEvent("on" + event, listenerFunc);
    }
};

/**
 * This method provides a browser neutral way of removing event listeners
 * from an element.  If the element supports the "removeEventListener" method
 * then this method is used.  Otherwise, if the element supports the
 * "detachEvent" method then this method is used.  There are no other fallback
 * methods so if neither of these methods exist then the event listener
 * is not detached.
 * 
 * @param {HTMLElement} elem the element from which an event listener
 *          is being removed (e.g. window, document, document.body)
 * @param {String} event the name of the event WITHOUT the "on"
 *          (e.g. "click", "resize", "mouseup")
 * @param {function} listenerFunc the event listener function
 */
Event.stopObserving = function(elem, event, listenerFunc, capture)
{
    if (elem.removeEventListener)
    {
        if (!capture)
        {
            capture = false;
        }
        
        elem.removeEventListener(event, listenerFunc, capture);
        return;
    }
    
    if (elem.detachEvent)
    {
        elem.detachEvent("on" + event, listenerFunc);
        return;
    }
};

Event.getTarget = function(event)
{
   return event.srcElement? event.srcElement : event.target; 
};