/**
 * A base class for grids that are NOT CPB Components.  Adds the following niceties:
 * 
 * 1. When paging and sorting using a JsonRest store, the grid keeps its size
 *    as opposed to rapidly shrinking/growing while fetching data.
 * 2. If a page selector is visible via the Pagination mixin, it is set to the
 *    proper value for the initial row count (which isn't done by Pagination by default!).
 */
define(
      "sas-cpb/datagrid/BaseGrid", ['sas-cpb/cpb', 'sas-cpb/menu/CPBMenu',
       'dijit/registry',
       'dgrid/Grid', 'dgrid/selector', 'dgrid/extensions/Pagination', 'dgrid/extensions/ColumnResizer',
       'dgrid/extensions/DijitRegistry',
       'dojo/_base/declare', 'dojo/_base/array', 'dojo/_base/lang', 'dojo/query', 'dojo/aspect', 'dojo/on',
       'dojo/dom-attr', 'dojo/domReady!'],
      function(cpb, CPBMenu,
            registry,
            Grid, selector, Pagination, ColumnResizer, DijitRegistry,
            declare, array, lang, query, aspect, on,
            domAttr) {

   var BaseGrid = declare('BaseGrid', [Grid, Pagination, ColumnResizer], {
   
      pageSizeStorageName : '',
      pageNumberStorageName : '',
      formName : '',
      
      constructor: function(args) {
         lang.mixin(this, args);
         
         // Ensure any page size options are sorted, and the initial row count
         // is included in it.
         if (this.pageSizeOptions) {
            if (this.rowsPerPage &&
                  array.indexOf(this.pageSizeOptions, this.rowsPerPage) === -1) {
               this.pageSizeOptions.push(this.rowsPerPage);
            }
            this.pageSizeOptions.sort(function(a, b) { return a - b; });
         }                  
      },
   
      postCreate: function() {
         
         this.inherited(arguments);
         
         var grid = this;
         this.on('dgrid-refresh-complete', function(evt) {
            // Let grid resize to whatever's appropriate for its data
            //grid._fixHeight(false);
            if (grid._isSorting) {
               delete grid._isSorting;
            }
            // Resize required to render RHS "scrollbar" area correctly on first render
            grid.resize();
         });
         
         // Hack to make page size selector default to our initial page size
         var pageSizeSelector = this._getPageSizeSelector();
         if (this.rowsPerPage && pageSizeSelector) {
            pageSizeSelector.value = this.rowsPerPage;
            pageSizeSelector.ariaLabel = AppProperties['table.rowCount.select.tip'];
         }
         
         //Add hook to store the pageSize if it is changed.
         on(pageSizeSelector, 'change', function(evt) {
            if (!BaseGrid.pageSizeStorageName)
               BaseGrid.pageSizeStorageName = BaseGrid.formName + '_pageSize';
            sessionStorage[BaseGrid.pageSizeStorageName] = grid.rowsPerPage; 
            
            // Reset the page number to 1 when the page size changes.  This is what Pagination does anyway, we just
            // need to make sure the session storage value matches.
            if (!BaseGrid.pageNumberStorageName)
               BaseGrid.pageNumberStorageName = BaseGrid.formName + '_pageNumber';
            sessionStorage[BaseGrid.pageNumberStorageName] = 1;
         });      
         
         // Cache the total row count from the store
         aspect.after(grid, 'renderArray', function(results) {
            if (typeof results.total === 'number') {
               grid._fixHeight(false);
               grid.resize();
            }
            else {
               results.total.then(function(results) {
                  grid._fixHeight(false);
                  grid.resize();
               });
            }
         }, true);
         
         if (document.body.dir === 'rtl') {
            BaseGrid.fixDgridRtlBugs(this);
         }
         BaseGrid.fixA11yBugs(this);
         
      },
      
      _destroyChildrenRecursively: function() {
         var widgets = registry.findWidgets(this.domNode);
         array.forEach(widgets, function(widget) {
            widget.destroyRecursive(false);
         });
      },
      
      _fixHeight: function(fix) {
         // Check to see if we're in the DOM yet.  This could be initial display now that we save page number.
         if (this.domNode) {
            var scrollers = query('.dgrid-scroller', this.domNode);
            if (scrollers && scrollers.length > 0) {
               var scroller = scrollers[0];
               var height = fix ? dojo.position(scroller).h + 'px' : '';
               scroller.style.minHeight = height;
            }
         }
      },
      
      _getPageSizeSelector: function() {
         var selects = query('.dgrid-page-size', this.domNode);
         // Should be 0 or 1
         return selects && selects.length > 0 ? selects[0] : null;
      },
      
      gotoPage: function(page, focusLink) {
         // Don't shrink the grid height while fetching more data from server
         this._fixHeight(true);
         this._destroyChildrenRecursively();
        
         if ((this._sort[0] && page == 1) || focusLink || page != 1 || (this._isSorting == undefined && page == 1) ||
               sessionStorage[BaseGrid.pageNumberStorageName] == null) {
            //Setting page on session when we change pages
            if (!BaseGrid.pageNumberStorageName)
               BaseGrid.pageNumberStorageName = BaseGrid.formName + '_pageNumber';               
            sessionStorage[BaseGrid.pageNumberStorageName] = page; 
             
            return this.inherited(arguments); 
         }
         else { //Setting saved page number when creating and not sorting - Pagination refresh always sets page to 1.
            if (this._sort[0] == undefined && page == 1 && sessionStorage[BaseGrid.pageNumberStorageName]) {
               arguments[0] = parseInt(sessionStorage[BaseGrid.pageNumberStorageName]);
               return this.inherited(arguments);
            }           
         }        
      },
      
      _setSort: function(property, descending) {
         if (this._isSorting) { // dgrid issue #635
            return;
         }         
         
         this._isSorting = true;
         // Don't shrink the grid height while fetching more data from server
         this._fixHeight(true);
         this._destroyChildrenRecursively();
         this.inherited(arguments);
      }
      
   });
   
   // summary:
   //   Fixes random RTL issues in dgrid that should really be done
   //   with CSS in the theme.  This is shared so non-BaseGrid dgrids can
   //   consume it also.
   BaseGrid.fixDgridRtlBugs = function(grid) {
      require(['dojo/dom-style', 'dojo/query'], function(domStyle, query) {
         
         // Column selector doesn't move to the left
         var colSelector = query('.dgrid-header-scroll', grid.domNode);
         if (colSelector && colSelector.length>0) {
            domStyle.set(colSelector[0], 'right', 'auto');
            domStyle.set(colSelector[0], 'left', '0');
         }
     
         // Stuff in the footer isn't mirrored
         var statusText = query('.dgrid-status', grid.domNode);
         if (statusText && statusText.length>0) { // Should always === 1
            domStyle.set(statusText[0], 'float', 'right');
         }
         var pageSizer = query('.dgrid-page-size', grid.domNode);
         if (pageSizer && pageSizer.length>0) {
            domStyle.set(pageSizer[0], 'float', 'left');
         }
         var navLinks = query('.dgrid-navigation', grid.domNode);
         if (navLinks && navLinks.length>0) {
            domStyle.set(navLinks[0], 'float', 'left');
         }
         // Column selector dropdown menu doesn't move to the left
         var colSelector = query('.dgrid-hider-menu', grid.domNode);
         if (colSelector && colSelector.length>0) {
            domStyle.set(colSelector[0], 'left', '17px');
            domStyle.set(colSelector[0], 'right', 'auto');
         }
         
      });
   };
   
   // summary:
   //   Fixes random a11y issues in dgrid.  This is shared so non-BaseGrid dgrids can
   //   consume it also.
   BaseGrid.fixA11yBugs = function(grid) {
      
      // Make the "x-y of z results" text non-focusable (dgrid issue #848)
      query('.dgrid-status', grid.domNode).forEach(function(elem) {
         elem.removeAttribute('tabindex');
      });
      
      // Make disabled pagination links non-focusable (also dgrid issue #848)
      aspect.after(grid, '_updateNavigation', function() {
         var disabledNavSpans = query('.dgrid-page-disabled', grid.domNode);
         disabledNavSpans.forEach(function(elem) {
            elem.removeAttribute('tabindex');
         });
      });
      
   };
   
   return BaseGrid;
   
});
