/**
 *  The grid component at the core of ActionableGrid.  Should be considered private.
 *  
 *  This grid component offers the following niceties:
 *  
 * 1. When paging and sorting using a JsonRest store, the grid keeps its size
 *    as opposed to rapidly shrinking/growing while fetching data.
 * 2. If a page selector is visible via the Pagination mixin, it is set to the
 *    proper value for the initial row count (which isn't done by Pagination by default!).
 * 3. Other miscellaneous dgrid bugs fixed.
 * 4. Handy method to mask/unmask the grid while it is performing an operation.
 * 5. Ability to refresh the grid's display, optionally flushing the server-side store as well.
 */
// TODO: For historical reasons, this was separate from BaseGrid.  There is no reason this
// shouldn't extend the other.  BaseGrid is purposely kept simpler, but there should be better
// code reuse.
define(
      "sas-cpb/datagrid/_BaseComponentGrid", ['sas-cpb/cpb', 'sas-cpb/datagrid/BaseGrid', 'sas-cpb/datagrid/RefreshingGridTracker',
       'sas-cpb/datagrid/CPBDataGrid', 'sas-cpb/messages/Messages',
       'dgrid/Grid', 'dgrid/Selection', 'dgrid/extensions/Pagination', 'dgrid/extensions/ColumnResizer',
       'dgrid/extensions/ColumnHider', 'dgrid/extensions/ColumnReorder', 'dgrid/extensions/DijitRegistry',
       'dojox/widget/Standby',
       'dojo/_base/declare', 'dojo/_base/lang', 'dojo/_base/array', 'dojo/query', 'dojo/dom-construct', 'dojo/aspect',
       'dojo/dom-style', 'dojo/has', 'dojo/on', 'dojo/domReady!'],
      function(cpb, BaseGrid, RefreshingGridTracker,
            CPBDataGrid, Messages,
            Grid, Selection, Pagination, ColumnResizer, ColumnHider, ColumnReorder, DijitRegistry,
            Standby,
            declare, lang, array, query, domConstruct, aspect,
            domStyle, has, on) {

var ctrlEquiv = has("mac") ? "metaKey" : "ctrlKey";

var BaseComponentGrid = declare('BaseComponentGrid',
      [Grid, Selection, Pagination, ColumnResizer, ColumnHider/*, ColumnReorder*/, DijitRegistry], {
   
   _ignoreSelectEvents: false,
   _cpb_standby: null,
   
   constructor: function(args) {
      if (args && args.fireOnSelectEvents) {
         // S1033958
         args.selectionEvents = 'click,dgrid-cellfocusin';
      }
      lang.mixin(args);
   },
   
   postCreate: function()  {  
      // We do this stuff here in postCreate because if we do it after dgrid creation,
      // it's too late for some stuff like initially-disabled row actions due to maxRows
      var grid = this;
      // Register all refreshListeners
      if (this.refreshListeners && this.refreshListeners.length > 0) {
         array.forEach(this.refreshListeners, function(listener) {
            grid.on('dgrid-refresh-complete', listener.onrefreshListener);
         });
      }
      grid.on('dgrid-error', function(error) {
         // Slow-loading grids may still be loading data on a page redirect.
         // In this case, don't show errors stemming from their canceled Ajax requests.
         if (!cpb.isRedirecting) {
            cpb.consoleLog(error.toString(), 'error');
            grid.cpb_showMessage(AppProperties['table.default.errorLoadingData.message.txt'], 'error');
         }
      });
      // Cache the total row count from the store
      aspect.after(grid, 'renderArray', function(results) {
         if (typeof results.total === 'number') {
            grid.cpb_totalRowCount = results.total;
            grid._fixHeight(false);
            grid.resize();
         }
         else {
            results.total.then(function(results) {
               grid.cpb_totalRowCount = results;
               grid._fixHeight(false);
               grid.resize();
            });
         }
      }, true);
      
      this.inherited(arguments);
      
      var pageSizeSelector;
      if (this._addAllPageSize) {
         pageSizeSelector = grid.cpb_getPageSizeSelector();
         var label = AppProperties['table.default.all.rows.selectChoice.txt'];
         domConstruct.create('option', { innerHTML: label, value: 1000000 }, pageSizeSelector);
      }
      delete this._addAllPageSize;
      if (this.pageSizeOptions && !this.printMode) {
         pageSizeSelector = grid.cpb_getPageSizeSelector();
         if (pageSizeSelector) {
            pageSizeSelector.value = grid.rowsPerPage;
            pageSizeSelector.ariaLabel = AppProperties['table.rowCount.select.tip'];
         }
      }
      
      grid.on('dgrid-refresh-complete', function(evt) {
         // Let grid resize to whatever's appropriate for its data
         //grid._fixHeight(false);
         if (grid._isSorting) {
            delete grid._isSorting;
         }
         grid.resize(); // Column/column header alignment issue on first render
      });

      // on-select events may also need to be triggered
      if (this.fireOnSelectEvents)
      {
         this._addOnSelectAdvice();
      }
      
      // dgrid doesn't have a "style" attribute like other dijits, so we must do this ourselves
      if (this.style) {
         //domStyle.set(this.domNode, this.style);
         delete this.style;
      }
      
      if (document.body.dir === 'rtl') {
         BaseGrid.fixDgridRtlBugs(grid);
      }
      BaseGrid.fixA11yBugs(this);
      
      CPBDataGrid.gridMap[grid.cpb_name] = this;
   },
   
   cpb_getPageSizeSelector: function() {
      var selects = query('.dgrid-page-size', this.domNode);
      // Should be 0 or 1
      return selects && selects.length>0 ? selects[0] : null;
   },
   
   cpb_getSelectedRowValues: function(colName) {
      colName = colName || this.cpb_selectedKeyField;
      var values = [];
      for (var id in this.selection) {
         // selection value may be "false" if using "select all" checkbox and individual checkboxes
         if (this.selection[id]) {
            var rowData = this.row(id);
            if (rowData.data[colName]!=null) { // Don't push null or undefined, but do push 0 and false
               values.push(rowData.data[colName]);
            }
         }
      }
      return values;
   },
   
   cpb_getAllRowValues: function(colName) {
      colName = colName || this.cpb_selectedKeyField;
      var values = [];
      this.store.query({}).forEach(function(record) {
         values.push(record[colName]);
      });
      return values;
   },
   
   cpb_refresh: function(flushStore) {
      flushStore = flushStore || false;
      
      if (!this._started) {
         cpb.consoleLog("Grid " + this.cpb_name + " not yet started; calling startup()...");
         this.startup();
      }
      else {
         cpb.consoleLog("Grid " + this.cpb_name + " started; refreshing...");
         
         if (flushStore) {
            CPBDataGrid.refreshServerStore(this.cpb_storeName);
         }
         else {
            var grid = this;
            RefreshingGridTracker.addRefreshingGrid(this);
            CPBService.clearGridData(this.cpb_name, cpb.getFormParams(), 
               function(isCleared)  {
                  CPBDataGrid.clearGridAndReload(grid.cpb_name);
                  if (grid.cpb_hasOnChangeEvents) {
                     cpb.fieldValueChanged(grid.cpb_name);
                  }
               }
            );
         }
      }
   },
   
   // summary:
   //    Shows a message in the UI related to this grid.
   //    "type" should be one of 'info', 'warning', or 'error', defaulting to 'info'.
   cpb_showMessage: function(message, type) {
      Messages.showMessage(message, type);
   },
   
   _fixHeight: function(fix) {
      var scroller = query('.dgrid-scroller', this.domNode)[0];
      var height = fix ? dojo.position(scroller).h + 'px' : '';
      scroller.style.minHeight = height;
   },
   
   gotoPage: function(page, focusLink) {
      // Don't shrink the grid height while fetching more data from server
      this._fixHeight(true);
      this._ignoreSelectEvents = true;
      this.clearSelection(); // Why is this needed?
      var result = this.inherited(arguments);
      this._ignoreSelectEvents = false;
      if (this.fireOnSelectEvents) { cpb.fieldSelected(this.id, {}); }
      return result;
   },

   _setSort: function(property, descending) {
      if (this._isSorting) { // dgrid issue #635
         return;
      }
      this._isSorting = true;
      // Don't shrink the grid height while fetching more data from server
      this._fixHeight(true);
      this._ignoreSelectEvents = true;
      this.inherited(arguments);
      this._ignoreSelectEvents = false;
      if (this.fireOnSelectEvents) { cpb.fieldSelected(this.id, {}); }
   },

   // _handleSelect overridden to prevent firinh two on-select events when the user clicks from one row
   // to another.  No good way to give advice, etc. for this.
   _handleSelect: function(event, currentTarget){
      // don't run if selection mode is none,
      // or if coming from a dgrid-cellfocusin from a mousedown
      if(this.selectionMode == "none" ||
            (event.type == "dgrid-cellfocusin" && event.parentType == "mousedown") ||
            (event.type == "mouseup" && currentTarget != this._waitForMouseUp)){
         return;
      }
      this._waitForMouseUp = null;
      this._selectionTriggerEvent = event;
      var ctrlKey = !event.keyCode ? event[ctrlEquiv] : event.ctrlKey;
      if(!event.keyCode || !event.ctrlKey || event.keyCode == 32){
         var mode = this.selectionMode,
            row = currentTarget,
            rowObj = this.row(row),
            lastRow = this._lastSelected;
         
         if(mode == "single"){
            if(lastRow === row){
               // Allow ctrl to toggle selection, even within single select mode.
               this.select(row, null, !ctrlKey || !this.isSelected(row));
            }else{
               // Don't fire on-select events if we're just clearing the selection for a new one
               this._cpb_skipFiringSelectEvents = true;
               this.clearSelection();
               delete this._cpb_skipFiringSelectEvents;
               this.select(row);
               this._lastSelected = row;
            }
         }else if(this.selection[rowObj.id] && !event.shiftKey && event.type == "mousedown"){
            // we wait for the mouse up if we are clicking a selected item so that drag n' drop
            // is possible without losing our selection
            this._waitForMouseUp = row;
         }else{
            var value;
            // clear selection first for non-ctrl-clicks in extended mode,
            // as well as for right-clicks on unselected targets
            if((event.button != 2 && mode == "extended" && !ctrlKey) ||
                  (event.button == 2 && !(this.selection[rowObj.id]))){
               this.clearSelection(rowObj.id, true);
            }
            if(!event.shiftKey){
               // null == toggle; undefined == true;
               lastRow = value = ctrlKey ? null : undefined;
            }
            this.select(row, lastRow, value);
   
            if(!lastRow){
               // update lastRow reference for potential subsequent shift+select
               // (current row was already selected by earlier logic)
               this._lastSelected = row;
            }
         }
         if(!event.keyCode && (event.shiftKey || ctrlKey)){
            // prevent selection in firefox
            event.preventDefault();
         }
      }
      this._selectionTriggerEvent = null;
   },

   _cpb_fixPaginationLinks: function() {
      // Fix dgrid pagination links
      // NOTE: This is fixed with dgrid 0.3.7, so remove this fix in the next release!
      var paginationLinks = query('a.dgrid-first, a.dgrid-previous, a.dgrid-page-link, a.dgrid-next, a.dgrid-last', this.domNode);
      if (paginationLinks && paginationLinks.length>0) {
         for (var i=0; i<paginationLinks.length; i++) {
            var a = paginationLinks[i];
            // Replace links with href='javascript:;' to href='#' and call e.preventDefault()
            if (!/#$/.test(a.href)) {
               a.href = '#';
               a.addEventListener('click', function(e) { e.preventDefault(); });
            }
         }
      }
   },
_updateNavigation: function() {
this.inherited(arguments);
this._cpb_fixPaginationLinks();
},

   showMask: function() {
      if (this._cpb_standby) {
         return; // Grid is already masked
      }
      var div = dojo.byId('gridAndToolbarDiv_' + this.cpb_name);
      if (div) { // i.e. the grid isn't hidden
         this._cpb_standby = new Standby({ target: div });
         document.body.appendChild(this._cpb_standby.domNode);
         this._cpb_standby.show();
      }
   },
   
   hideMask: function() {
      if (this._cpb_standby) {
         this._cpb_standby.hide();
         delete this._cpb_standby;
      }
   },
   
   // TODO: Remove when dgrid issue #749 is fixed
   _refreshAndKeepPage: function() {
      
      // Code below is ripped from Pagination.refresh(), just go to this._currentPage instead of page 1
      
      var self = this;
      
      // Reset to first page and return promise from gotoPage
      return this.gotoPage(this._currentPage).then(function(results){
         // Emit on a separate turn to enable event to be used consistently for
         // initial render, regardless of whether the backing store is async
         setTimeout(function() {
            on.emit(self.domNode, "dgrid-refresh-complete", {
               bubbles: true,
               cancelable: false,
               grid: self,
               results: results // QueryResults object (may be a wrapped promise)
            });
         }, 0);
         
         return results;
      });
   }, 

   _addOnSelectAdvice: function() {
      
      var grid = this;
      
      aspect.after(grid, 'select', function(row, toRow, value) {
         if (grid._ignoreSelectEvents) {
            return;
         }
         // See select() method for argument values and their meanings
         if(value === undefined){
            // default to true
            value = true;
         }
         if (!row.element) {
            row = this.row(row);
         }
         var selected = this.selection[row.id];
         var lastSelectedRowData = selected ? row.data : {};
         cpb.fieldSelected(grid.id, lastSelectedRowData);
      }, true);
      
      // dgrid will call select() for each row individually in a selectAll() call, but
      // we don't want to fire 20+ on-select events.  We only want to fire one.
      aspect.before(grid, 'selectAll', function() {
         grid._ignoreSelectEvents = true;
      });
      aspect.after(grid, 'selectAll', function() {
         grid._ignoreSelectEvents = false;
         // on-select is not technically supported for multi-select grids, but we'll be nice and return an
         // arbitrary selected value
         var selectedKey = null;
         for (var item in grid.selection) {
            selectedKey = item;
            break;
         }
         if (selectedKey) {
            cpb.fieldSelected(grid.id, grid.row(selectedKey).data);
         }
      });
      
      // dgrid will call deselect() for each row individually in a clearSelection() call, but
      // we don't want to fire 20+ on-select events.  We only want to fire one.
      aspect.before(grid, 'clearSelection', function() {
         grid._ignoreSelectEvents = true;
      });
      aspect.after(grid, 'clearSelection', function() {
         grid._ignoreSelectEvents = false;
         if (grid._cpb_skipFiringSelectEvents) {
            cpb.consoleLog('Not firing select events for grid as a new selection is imminent');
         }
         else {
            cpb.consoleLog('Selection cleared with no imminent new selection, firing on-select');
            cpb.fieldSelected(grid.id, {});
         }
      });
      
   }
   
});

return BaseComponentGrid;

});