/***********************************************************************************/
/* Copyright (c) 2014, SAS Institute Inc., Cary, NC, USA, All Rights Reserved      */
/*                                                                                 */
/* NAME:          ecm_srch_get_field_list                                          */
/* PRODUCT:       Enterprise Case Management                                       */
/* PURPOSE:       get the list of search fields and map the source                 */
/*                columns name or udf names to the search field names.             */
/* PARAMETERS:    object_nm=ECM entity type name.                                  */
/*                          CASE, INCIDENT, PARTY, RR, EFILE                       */
/*                srch_name_in_prefix=prefix of name_in field in search_config     */
/*                         it is used to filter search columns                     */
/*                         if blank, <udf_def_dsn>.UDF_TABLE_NM is used            */
/*                src_dsn=name of the table that contain columns for indexing      */
/*                      if blank, ECM_DB.<object_nm>_LIVE                          */
/*                       will be used.                                             */
/*                out_dsn=output table name that contains the mapping of           */
/*                        source column name to search column name                 */
/*                udf_def_dsn=name of the udf definition table                     */
/*                srchcfg_dsn=table containing search config field leve data.      */
/*                multi_row_YN=Y or N to indicate whether there are multiple       */
/*                            rows for per <object>.                               */
/* NOTE:                                                                           */
/* if &src_dsn and &udf_def_dsn are both empty, an empty table will be created.    */
/***********************************************************************************/
   %macro ecm_srch_get_field_list( 
                                     object_nm=
                                   , type_field_value=
                                   , srch_name_in_prefix=
                                   , src_dsn=
                                   , out_dsn=
                                   , udf_def_dsn=
                                   , srchcfg_dsn=ECM_RPT.SEARCH_CONFIG
                                   , multi_row_YN=
                                 );

   %let object_nm=%upcase(&object_nm);
   %let src_name_in_prefix=%upcase(&srch_name_in_prefix);
   %let src_dsn=%upcase(&src_dsn);
   %let out_dsn=%upcase(&out_dsn);
   %let udf_def_dsn=%upcase(&udf_def_dsn);
   %let multi_row_yn=%upcase(&multi_row_yn);


   %let abortRC=0;
   %put %sysfunc(sasmsg(&smdds,GEN_MAC_START_NOTE,NOQUOTE,&sysmacroname)) ;

     data &out_dsn;
          attrib
              table_struct_type length=$30 label='Tabe Structure Type'
              src_table_nm length=$62 label='Source Table Name'
              ecm_table_nm length=$30 label='ECM UDF table name'
              src_column_nm length=$30 label='Source column name'
              sas_datatype length=8 label='SAS data type'
              ecm_datatype length=$30 label='ECM UDF type'
              srch_field_nm length=$256 label='Search field name'
              srch_datatype length=$30 label='Search data type'
     ;
     stop;
     run;

     %if %str(&src_dsn) ne %str() %then 
     %do; /* src_dsn is not empty */

         %if %sysfunc(exist(&src_dsn))=0 and  %sysfunc(exist(&src_dsn,view))=0 %then 
         %do; /* src_dsn does not exist */

            %let abortRC=1;
             %put %sysfunc(sasmsg(&smdds,GEN_TABLE_MISSING_ERROR,NOQUOTE,&src_dsn)) ;
            %goto mEXIT; 

         %end; /* src_dsn does not exist */

         /* Retreive column information */
         proc contents data=&src_dsn out=_t_src_columns (keep=name type) noprint;
         run;

         /* Map source columns with search field info */
         proc sql noprint;
             create table _t_column_rect as 
             select  distinct
             %if &multi_row_YN=Y %then
             %do; 
                       "MULTI" as table_struct_type  length=30
             %end;
             %else 
             %do; 
                       "RECT" as table_struct_type  length=30
             %end;
                     , "&src_dsn" as src_table_nm length=62

                     %if %str(&src_dsn) = %str(ECM_DB.&object_nm._LIVE) %then
                     %do; /* if srcdsn is the ECM live table */
                     , "&object_nm"
                     %end;
                     %else
                     %do;
                     , " "
                     %end; 
                      as ecm_table_nm length=30
                     , upcase(a.name) as src_column_nm length=30
                     , b.name_in as srch_field_nm length=256
                     , a.type as sas_datatype 
                     , '' as ecm_datatype length=30
                     , upcase(b.datatype) as srch_datatype length=30
                     
             from _t_src_columns as a,
                  &srchcfg_dsn as b
             where    1
             %if %nrbquote(&type_field_value) ne %nrbquote() %then
             %do; 
                  and upcase(type_field_value)="%upcase(&type_field_value)"
             %end; 
             %if %str(&srch_name_in_prefix) ne %str() %then
             %do; /* if prefix is provided, join by prefix||name */
                  and upcase(a.name)=scan(upcase(b.name_in),2,'.')
                  and scan(upcase(b.name_in),1,'.')="&srch_name_in_prefix"
             %end; /* if prefix is provided, join by prefix||name */
             %else
             %if %str(&src_dsn) = %str(ECM_DB.&object_nm._LIVE) %then
             %do; /* if srcdsn is the ECM live table */
                  and upcase(a.name)=scan(upcase(b.name_in),2,'.')
                  and scan(upcase(b.name_in),1,'.')="&object_nm"
             %end; /* if prefix is provided, join by prefix||name */
             %else
             %do; /* if prefix is not provided and src_dsn is not ECM live table, join by column names */
                  and upcase(a.name)=upcase(b.name_in)
             %end; /* if prefix is not provided and src_dsn is not ECM live table, join by column names */
                             
                             
            ;
        quit;

        proc append base=&out_dsn data=_t_column_rect force;
        run;

     %end; /* src_dsn is not empty */

     %if %str(&udf_def_dsn) ne %str() %then 
     %do; /* udf_def_dsn is not empty */

         %if %sysfunc(exist(&udf_def_dsn))=0 %then 
         %do; /* src_dsn does not exist */

            %let abortRC=1;
             %put %sysfunc(sasmsg(&smdds,GEN_TABLE_MISSING_ERROR,NOQUOTE,&udf_def_dsn)) ;
            %goto mEXIT; 

         %end; /* udf_def_dsn does not exist */

          proc sql noprint;
             create table _t_column_long as 
             select  distinct
                       "LONG" as table_struct_type  length=30                  
                     /* store a portion of the UDF source table name and the complete name will be derived by the calling program */
                     /* as ECM_DB.<object_nm>._UDF_<src_table_nm>_VALUE                                                           */
                     , case
                         when udf_type_nm in ('TIMESTAMP' 'DATE') then "DATE"
                         when udf_type_nm in ('INTEGER' 'DOUBLE' 'BIGINIT') then "NUM"
                         when udf_type_nm in ('LNGVARCHAR') then "LGCHR"
                         else "CHAR"
                       end as  src_table_nm  length=62                     
                     , upcase(a.udf_table_nm) as ecm_table_nm length=30
                     , upcase(a.udf_nm) as src_column_nm length=30
                     , b.name_in as srch_field_nm length=256
                     , . as sas_datatype
                     , upcase(a.udf_type_nm) as ecm_datatype 
                     , upcase(b.datatype) as srch_datatype length=30
                     
             from &udf_def_dsn as a,
                  &srchcfg_dsn as b
               
             where    1
             %if %nrbquote(&type_field_value) ne %nrbquote() %then
             %do; 
                  and upcase(type_field_value)="%upcase(&type_field_value)"
             %end;  
                   and upcase(a.udf_table_nm)=scan(b.name_in,1,'.')
                   and upcase(a.udf_nm)=scan(upcase(b.name_in),2,'.')     
             %if %str(&srch_name_in_prefix) ne %str() %then
             %do; 
                   and scan(b.name_in,1,'.')="&srch_name_in_prefix"
             %end;                      
                             
            ;
        quit;

        proc append base=&out_dsn data=_t_column_long force nowarn;
        run;

    %end; /* udf_def_dsn is not empty */

    
quit;          
          

   
%mEXIT:

%put %sysfunc(sasmsg(&smdds,GEN_MAC_END_NOTE,NOQUOTE,&sysmacroname)) ;
%mend;
 
