/***********************************************************************************/
/* Copyright (c) 2014, SAS Institute Inc., Cary, NC, USA, All Rights Reserved      */
/*                                                                                 */
/* NAME:          ecm_srch_prep_src_data                                           */
/* PRODUCT:       Enterprise Case Management                                       */
/* PURPOSE:       set up the source data for search indexing.                      */
/*                columns name or udf names to the search field names.             */
/* PARAMETERS:    object_nm=ECM entity type name.                                  */
/*                          CASE, INCIDENT, PARTY, RR, EFILE                       */
/*                input_type=input_type parameter from ECM_SRCH_INDEX_DRIVER.      */
/*                           The valid values are FROM_DTTM, SUBSET_DSN or SRC_DSN */
/*                from_dttm=from_dttm parameter from ECM_SRCH_INDEX_DRIVER.        */
/*                subset_dsn=subset_dsn parameter from ECM_SRCH_INDEX_DRIVER.      */
/*                src_dsn=name of the table that contain columns for indexing      */
/*                      if blank, ECM_DB.<object_nm>_LIVE                          */
/*                       will be used.                                             */
/*                multi_row_YN= indicate whether the src_dsn contain multiple rows */
/*                        per object. Not supported yet.                           */
/*                out_list_dsn=output table name that contains the list of         */
/*                        source data tables.                                      */
/*                udf_list_dsn=name of the udf definition table                    */
/*                use_ecm_tmp_YN= Y or N to use ECM_TMP library for more efficient */
/*                                more processing.                                 */
/*                               It requires proper set up of DBMS and SAS         */
/* NOTE:                                                                           */
/* if &src_dsn and &udf_list_dsn are both empty, an empty table will be created.   */
/***********************************************************************************/

   %macro ecm_srch_prep_src_data (  object_nm= 
                                  , input_type=
                                  , from_dttm=
                                  , subset_dsn=
                                  , src_dsn=
                                  , multi_row_YN=N 
                                  , udf_list_dsn=
                                  , out_list_dsn=
                                  , use_ecm_tmp_YN=N
                                 );

   
   %let abortRC=0;
   %if &use_ecm_tmp_YN=Y %then %let tmp_lib=ECM_TMP;
   %else %let tmp_lib=WORK;
   
   %put %sysfunc(sasmsg(&smdds,GEN_MAC_START_NOTE,NOQUOTE,&sysmacroname)) ;

/* macro for generating user group data */

%macro ecm_srch_prep_group(object_rk_dsn=);          
proc sql;
     create table &object_nm._x_user_group as
     select   distinct a.*
            , '2' as udf_table_nm length=90
            , ' ' as udf_nm length=90
     from   ecm_db.&object_nm._x_user_group as a
          , &object_rk_dsn as b
     where a.&object_nm._rk=b.&object_nm._rk
     order by    &object_nm._rk
     ; 
     %let out_list_cnt=%eval(&out_list_cnt+1);
     insert into &out_list_dsn 
           values ("ECM_DB.&object_nm._X_USER_GROUP", "&object_nm._X_USER_GROUP",&out_list_cnt,'GROUP');

%mend;

%local out_list_cnt 
       rect_cnt
       long_dsn_cnt
;
%let rect_cont=0;
%let out_list_cnt=0;

/* Define the structure of the output table */
data &out_list_dsn;
     attrib orig_src_dsn length=$62 /* SRC_TABLE_NM from &udf_list_dsn */
            src_dsn  length=$62
            order    length=8
            src_type length=$30
     ;
     stop;
run; 

%if &input_type = SRC_DSN %then %let long_dsn_cnt=0;
%else
%do;

/* Get a portion of the UDF source table name (DATE, NUM, LGHCR or CHAR) */
/* It will be used to make up different column name for UDF_VALUE in different */ 
/*  <object>_UDF_<>_VALUE tables */
proc sql noprint;
    select distinct  src_table_nm                 
           into
               :long_dsn_1 - 
    from &udf_list_dsn
    where table_struct_type='LONG'
;
%let long_dsn_cnt=&sqlobs;
quit;
%end;

%if &input_type=FROM_DTTM %then 
%do; /* filter by from_dttm */
     
     
      %ecm_table_drop(dsn=&object_nm._live);
      proc sql;
           create table &object_nm._live           
           as
           select * 
                  , '1' as udf_table_nm length=90
                  , ' ' as udf_nm length=90
           from ecm_db.&object_nm._live
           where valid_from_dttm >= &from_dttm
           order by &object_nm._rk
           
      ;
      %let rect_obs=&sqlobs;

      %if &rect_obs > 0 %then
      %do;
 
         %ecm_srch_prep_group(object_rk_dsn=&object_nm._LIVE); 
         %let out_list_cnt=%eval(&out_list_cnt+1);
          insert into &out_list_dsn 
              values ("ECM_DB.&object_nm._LIVE", "WORK.&object_nm._LIVE",&out_list_cnt,'RECT');
      %end;
      %else %let long_dsn_cnt=0;
        ;
%end; /* filter by from dttm */

%else %if &input_type=SUBSET_DSN %then 
%do; /* filter by from_dttm */
      
      %ecm_table_drop(dsn=&object_nm._live);

      proc sql;
           create table &object_nm._live 
           as
           select * 
                  , '1' as udf_table_nm length=90
                  , ' ' as udf_nm length=90
           from ecm_db.&object_nm._live 
           where &object_nm._rk in 
                 ( select distinct &object_nm._rk 
                   from &subset_dsn
                  )
           order by  &object_nm._rk
                     ;

         %let rect_obs=&sqlobs;

      %if &rect_obs > 0 %then
      %do;

        %ecm_srch_prep_group(object_rk_dsn=&object_nm._LIVE); 

           insert into &data_list_dsn 
           values ("ECM_DB.&object_nm._LIVE", "WORK.&object_nm._LIVE",%eval(&out_list_cnt+1),'RECT');
        ;
        quit;

        %if &syserr > 4 %then 
        %do;
             %let abortRC=1;
             %put %sysfunc(sasmsg(&smdds,GEN_MAC_ABORT,NOQUOTE,&sysmacroname)) ;
             %goto MEXIT;
        %end;

      %end;
      %else %let long_dsn_cnt=0;
      


%end; /* filter by from_dttm */
%else %if &input_type=SRC_DSN %then
%do;

      
     %if %str(&src_dsn) ne %str() %then
     %do;

          %let rect_obs=%ecm_nobs(&src_dsn);

          %if &rect_obs > 0 or &rect_obs=-1 %then
          %do;

         %ecm_srch_prep_group(object_rk_dsn=&src_dsn); 

         %if &multi_row_YN=N %then
         %do; /* one record per object */

             /* Create a view of the source table with UDF_TABLE_NM and UDF_NM.  */
             /* Assuming that src table has only one row per object. It is treated as rectangler table*/

             proc sql ;
                  create view _T_SRC_VIEW as
                   select * 
                          , '1' as udf_table_nm length=90
                          , ' ' as udf_nm length=90
                   from &src_dsn
             ;
            
             insert into &out_list_dsn 
                   values ("&src_dsn", "_T_SRC_VIEW", %eval(&out_list_cnt+1),'RECT');
                ;
              quit;

              
               %if &syserr > 4 %then 
               %do;
                     %let abortRC=1;
                     %put %sysfunc(sasmsg(&smdds,GEN_MAC_ABORT,NOQUOTE,&sysmacroname)) ;
                     %goto MEXIT;
               %end;

            %end; /* one record per object */
            %else 
            %do;  /* multiple records per object */


                            
            /* create a new temp file if it does not exist */
            %if not %sysfunc(fexist(tmp2)) %then 
            %do;
                filename tmp2 temp;
            %end;

            %let num_field_list=;
            %let char_field_list=;
            data _null_;
                file tmp2;    
                length char_field_list num_field_list $ 32767;
                retain char_field_list  ' '
                       num_field_list ' ' 
                       ;
                set &udf_list_dsn end=eof;
                length sas_stmt $ 32760;                
                sas_stmt='udf_nm="'||strip(src_column_nm)||'";';
                put sas_stmt;
                if sas_datatype=1 then
                do;
                  num_field_list=strip(num_field_list)||' '||strip(src_column_nm);
                  sas_stmt='udf_value_num='||strip(src_column_nm)||';';
                end;
                else
                do; 
                   char_field_list=strip(char_field_list)||' '||strip(src_column_nm);
                   sas_stmt='udf_value_char='||strip(src_column_nm)||';';
                end;
                put sas_stmt;
                sas_stmt='sas_datatype='||put(sas_datatype,1.)||';';
                put sas_stmt;
                sas_stmt='srch_field_nm="'||strip(srch_field_nm)||'";';
                put sas_stmt;
                sas_stmt='srch_datatype="'||strip(srch_datatype)||'";';
                put sas_stmt;

                sas_stmt='if udf_value_char ne "" or udf_value_num ne . then output;';
                put sas_stmt;
                if eof then 
                do;
                  call symput('num_field_list',strip(num_field_list));
                  call symput('char_field_list',strip(char_field_list));
                end;
                run;

                data MULTI_ROW (keep=&object_nm._rk 
                                     udf_table_nm udf_nm udf_value_char udf_value_num row_no
                                     sas_datatype srch_field_nm srch_datatype
                               ) 
                               ;
                     if 0 then set &udf_list_dsn (keep=sas_datatype srch_field_nm srch_datatype);
                     length udf_table_nm $ 30
                            udf_nm       $ 30
                     ; 
                     retain udf_table_nm '1';
                     set &src_dsn;
                     by &object_nm._rk;
                     if first.&object_nm._rk then row_no=0;
                     row_no+1; 
                     %inc tmp2;
                run;  
             proc sort data=multi_row ;
                  by &object_nm._rk udf_table_nm udf_nm  row_no;
             run; 
 
             proc sql;      
                insert into &out_list_dsn 
                   values ("&src_dsn", "MULTI_ROW", %eval(&out_list_cnt+1),'MULTI');
                ;
              quit;

            %end; /* multiple records per object */

        %end;
      
     %end;
%end;
 

%if &long_dsn_cnt > 0 %then
%do; 

    %if &tmp_lib=ECM_TMP %then
    %do;
	    %ecm_table_drop(dsn=ecm_tmp._t_&sysjobid._udf_list);
	    %ecm_table_drop(dsn=ecm_tmp._t_&sysjobid._rk_list);


	    proc sql noprint;
		/* copy the udf table to ECM_TMP to make the join with ECM_DB faster */
		create table ecm_tmp._t_&sysjobid._udf_list as
		select * 
		from &udf_list_dsn ;

		/* copy the live table to ECM_TMP to make the join with ECM_DB faster */
		create table ecm_tmp._t_&sysjobid._rk_list as
		select &object_nm._rk, valid_from_dttm 
		from &object_nm._live ;
	    run;
    %end;

    
%if &syserr > 4 %then 
%do;
     %let abortRC=1;
     %put %sysfunc(sasmsg(&smdds,GEN_MAC_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto MEXIT;
%end;

    proc sql ;

    %local udf_src_dsn;

    /* Subset UDF value tables if long_dsn_cnt > 0 */

    %do i_long=1 %to &long_dsn_cnt;
          
          %let udf_src_dsn=ECM_DB.&object_nm._UDF_&&&long_dsn_&i_long.._VALUE;

          create table LONG_DSN_&i_long (index=(key2=(&object_nm._rk udf_table_nm udf_nm))) as
          select   val.&object_nm._rk
                 , val.valid_from_dttm
                 , val.udf_table_nm 
                 , val.udf_nm 
                 , val.udf_value as udf_value_&&&long_dsn_&i_long
                 , val.row_no
                 , def.srch_field_nm
                 , def.srch_datatype
                 , def.ecm_datatype 
          from     &udf_src_dsn as val
          %if &tmp_lib=ECM_TMP %then
          %do;
                ,  &tmp_lib.._t_&sysjobid._rk_list as rk
          %end;
          %else 
          %do;
                 ,  &tmp_lib..&object_nm._live (keep=&object_nm._rk valid_from_dttm) as rk 
          %end;
                ,  &tmp_lib.._t_&sysjobid._udf_list as  def     
          where     val.&object_nm._rk=rk.&object_nm._rk
                 and val.valid_from_dttm=rk.valid_from_dttm
                 and val.udf_table_nm=def.ecm_table_nm
                 and val.udf_nm=def.src_column_nm 
         order by    &object_nm._rk
                   , udf_table_nm
                   , udf_nm
                   , row_no 
         ;

         %let out_list_cnt=%eval(&out_list_cnt+1);
         insert into &out_list_dsn 
                  values ("&&&long_dsn_&i_long.", "LONG_DSN_&i_long",&out_list_cnt,"LONG");
            ;
    %end; /* loop thru UDF value tables */
    quit;


%if &syserr > 4 %then 
%do;
     %let abortRC=1;
     %put %sysfunc(sasmsg(&smdds,GEN_MAC_ABORT,NOQUOTE,&sysmacroname)) ;
     %goto MEXIT;
%end;

%end;
   
%mEXIT:

%put %sysfunc(sasmsg(&smdds,GEN_MAC_END_NOTE,NOQUOTE,&sysmacroname)) ;
%mend;
